﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Windows.Forms;


/**
 * TODO:
 * ・適切なコメントを書く
 * ・基本情報として年齢欄を追加する
 * ・更新ボタン押下時の入力値のチェック（特に, が含まれたデータには注意。身長、体重の値域チェックも忘れずに。データが不正なときは更新処理をしない）
 * ・新規作成だけではなく、顧客データを消す機能もつけてみる（データを消すときは、確認ダイアログが出たほうが親切 -> MessageBox.Show）
 * ・ファイル読み込み時のチェック（double.Parseが失敗するときの処理 -> try, cache と double.tryParse を使う）
 * ・ウィンドウの大きさが可変である必要はない。ウィンドウの大きさを固定にするオプションを調べて変更する
 * ・名前順にソートして表示 (and 保存)
 * ・LINQを使ってコードをきれいに整形
 * ・新規作成時に一時的に BMI: NaN, 状態: やせ になるのはあまり良くなさそう
 * ・新規作成専用のウィンドウを別に作り、それらを分離する
 * ・更新ボタンの仕様があまり良くない（編集したデータというよりも全体を保存し直す仕様になっている）
 * ・身長と体重欄で入力できる文字の種類を数字に限定する
 * ・Bindingを使ってupdate処理を消す
 */

namespace BMI_Recorder
{
    public partial class BMIRecoderMain : Form
    {
        private string databaseFilepath = @"customer.txt";
        public List<Customer> customerData = new List<Customer>();


        public BMIRecoderMain()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            ReadCustomerDataFromFile();
            UpdateCustomerList();
        }

        private void UpdateCustomerList()
        {
            customerList.Items.Clear();
            foreach (var customer in customerData)
            {
                customerList.Items.Add(customer);
            }
        }

        private void ReadCustomerDataFromFile()
        {
            customerData.Clear();
            var lines = File.ReadAllLines(databaseFilepath);
            foreach (var line in lines)
            {
                if (line == "") continue;
                var data = line.Split(',');
                if (data.Length != 3) continue;

                var customer = new Customer
                {
                    name = data[0],
                    height = double.Parse(data[1]),
                    weight = double.Parse(data[2])
                };
                customerData.Add(customer);
            }
        }

        private void SaveCustomerDataToFile()
        {
            var savedata = new List<string>();
            foreach (var customer in customerData)
            {
                savedata.Add(customer.ToDatabaseString());
            }
            File.WriteAllLines(databaseFilepath, savedata);
        }

        private void CustomerList_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (customerList.SelectedIndex == -1) return;

            var customerSelected = customerData[customerList.SelectedIndex];

            nameTextBox.Text = customerSelected.name;
            heightTextBox.Text = $"{customerSelected.height}";
            weightTextBox.Text = $"{customerSelected.weight}";
            BMIValue.Text = $"{customerSelected.BMI}";
            stateLabel.Text = customerSelected.state;
        }

        private void UpdateButton_Click(object sender, EventArgs e)
        {
            var index = customerList.SelectedIndex;
            if (index == -1) return;

            var customerSelected = customerData[index];
            customerSelected.name = nameTextBox.Text;
            customerSelected.height = int.Parse(heightTextBox.Text);
            customerSelected.weight = int.Parse(weightTextBox.Text);

            UpdateCustomerList();
            SaveCustomerDataToFile();

            customerList.SelectedIndex = index;
        }

        private void AddButton_Click(object sender, EventArgs e)
        {
            customerData.Add(new Customer());

            UpdateCustomerList();
            customerList.SelectedIndex = customerList.Items.Count - 1;
        }
    }

    public class Customer
    {
        public string name;
        public double height;
        public double weight;
        public double BMI
        {
            get
            {
                var h = height / 100;
                return Math.Truncate(weight / (h * h) * 100.0) / 100.0;
            }
        }
        public string state
        {
            get
            {
                if (BMI >= 25) {
                    return "肥満";
                } else if (BMI >= 18.5)
                {
                    return "標準";
                } else
                {
                    return "やせ";
                }
            }
        }
        public string ToDatabaseString()
        {
            return $"{name},{height},{weight}\n";
        }
        public override string ToString()
        {
            return $"{name}さん";
        }
    }
}
